@@ -2,7 +2,7 @@
 /*
  *  Memory allocation helpers.
  *
- *  Copyright (c) by Jaroslav Kysela <perex@suse.cz>
+ *  Copyright (c) by Jaroslav Kysela <perex@perex.cz>
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -157,7 +157,6 @@ int __init snd_memory_info_init(void)
 
 	entry = snd_info_create_module_entry(THIS_MODULE, "meminfo", NULL);
 	if (entry) {
-		entry->c.text.read_size = 256;
 		entry->c.text.read = snd_memory_info_read;
 		if (snd_info_register(entry) < 0) {
 			snd_info_free_entry(entry);
@@ -170,7 +169,6 @@ int __init snd_memory_info_init(void)
 
 void __exit snd_memory_info_done(void)
 {
-	if (snd_memory_info_entry)
-		snd_info_unregister(snd_memory_info_entry);
+	snd_info_free_entry(snd_memory_info_entry);
 }
 #endif
