@@ -9,10 +9,13 @@
 #endif
 
 #include "config.h"
-#undef CONFIG_SND_DEBUG_MEMORY
+#define SKIP_HIDDEN_MALLOCS
 #include "adriver.h"
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 0)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,23)
+#define CONFIG_HAS_DMA 1
+#endif
 #include "../alsa-kernel/core/sgbuf.c"
 #else
 
