@@ -6,9 +6,5 @@
 #define CONFIG_ADB_PMU
 #endif
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
-#define PMAC_SUPPORT_PCM_BEEP
-#endif
-
 #include "../alsa-kernel/ppc/powermac.c"
 EXPORT_NO_SYMBOLS;
