@@ -1,6 +1,6 @@
 /*
  *  Utility to find module dependencies from Modules.dep
- *  Copyright (c) by Jaroslav Kysela <perex@suse.cz>
+ *  Copyright (c) by Jaroslav Kysela <perex@perex.cz>
  *		     Anders Semb Hermansen <ahermans@vf.telia.no>,
  *		     Martin Dahl <dahlm@vf.telia.no>,
  *
@@ -53,6 +53,8 @@ struct sel {
 	struct sel *next;
 };
 
+enum { TYPE_TRISTATE, TYPE_BOOL, TYPE_INT };
+
 struct dep {
 	char *name;
 	// dependency part - conditions (chain)
@@ -62,7 +64,8 @@ struct dep {
 	// misc
 	struct dep *next;
 	// bool?
-	int is_bool;
+	int type;
+	int int_val;
 	// hitflag?
 	int hitflag;
 };
@@ -133,7 +136,8 @@ static char *kernel_deps[] = {
 	"VIDEO_DEV",
 	"VIDEO_V4L1",
 	"FW_LOADER",
-	/* some flags */
+	/* some flags/capabilities */
+	"HAS_IOPORT",
 	"EXPERIMENTAL",
 	/* sound common */
 	"AC97_BUS",
@@ -144,6 +148,7 @@ static char *kernel_deps[] = {
 /* # -> define */
 static char *no_cards[] = {
 	"%#SOUND",
+	"%HAS_IOMEM",
 	"SOUND_PRIME",
 	"%SND",
 	"#SND_TIMER",
@@ -165,6 +170,8 @@ static char *no_cards[] = {
 	"SND_VERBOSE_PROCFS",
 	"SND_VERBOSE_PRINTK",
 	"SND_BIT32_EMUL",
+	"#SND_AD1848_LIB",
+	"#SND_CS4231_LIB",
 	"#SND_SB_COMMON",
 	"#SND_SB8_DSP",
 	"#SND_SB16_DSP",
@@ -181,6 +188,7 @@ static char *no_cards[] = {
 	"#SND_SOC_WM8750",
 	"#SND_SOC_WM8753",
 	"#SND_SOC_WM9712",
+	"#SND_SOC_CS4270",
 	"#SND_PXA2XX_PCM",
 	"#SND_PXA2XX_AC97",
 	"#SND_PXA2XX_SOC_AC97",
@@ -326,15 +334,25 @@ static int read_file_1(const char *filename, struct cond **template)
 		switch (state) {
 		case READ_STATE_CONFIG:
 			if (!strncmp(buffer, "\ttristate", 9))
-				dep->is_bool = 0;
+				dep->type = TYPE_TRISTATE;
 			else if (!strncmp(buffer, "\tbool", 5))
-				dep->is_bool = 1;
+				dep->type = TYPE_BOOL;
 			else if (!strncmp(buffer, "\tdepends on ", 12))
 				add_dep(dep, buffer + 12, *template);
 			else if (!strncmp(buffer, "\tdepends ", 9))
 				add_dep(dep, buffer + 9, *template);
 			else if (!strncmp(buffer, "\tselect ", 8))
 				add_select(dep, buffer + 8, *template);
+			else if (!strncmp(buffer, "\tint ", 5))
+				dep->type = TYPE_INT;
+			if (!strncmp(buffer, "\tdefault ", 9)) {
+				if (dep->type == TYPE_INT) {
+					char *p = buffer + 9;
+					for (; *p && !isdigit(*p); p++)
+						;
+					dep->int_val = strtol(p, NULL, 0);
+				}
+			}
 			continue;
 		case READ_STATE_MENU:
 			if (!strncmp(buffer, "\tdepends ", 9)) {
@@ -408,6 +426,12 @@ static struct cond * create_cond(char *line)
 			for (i = 1; i < strlen(word) + 1; i++)
 				word[i - 1] = word[i];
 			cond->left++;
+			if (!word[0]) {
+				if (!get_word(line, word)) {
+					fprintf(stderr, "Unbalanced open-parenthesis\n");
+					exit(EXIT_FAILURE);
+				}
+			}
 		}
 		/* hack for !XXX */
 		if (word[0] == '!' && isascii(word[1])) {
@@ -445,7 +469,11 @@ static struct cond * create_cond(char *line)
 				continue;
 			} else if (!strcmp(word, "!=n"))
 				continue;
-			else {
+			else if (*word == ')') {
+				for (i = 0; word[i]; i++)
+					if (word[i] == ')')
+						cond->right++;
+			} else {
 				fprintf(stderr, "Wrong condition %s\n", word);
 				exit(EXIT_FAILURE);
 			}
@@ -891,7 +919,7 @@ static int is_always_true(struct dep *dep)
 }
 
 // Print out ALL deps for firstdep (Cards, Deps)
-static void output_card_list(struct dep *firstdep, int space, int size, int tristate)
+static void output_card_list(struct dep *firstdep, int space, int size, int type)
 {
 	struct dep *temp_dep=firstdep;
 	char *card_name;
@@ -903,9 +931,7 @@ static void output_card_list(struct dep *firstdep, int space, int size, int tris
 	while(temp_dep) {
 		if (!is_toplevel(temp_dep))
 			goto __skip;
-		if (tristate && temp_dep->is_bool)
-			goto __skip;
-		if (!tristate && !temp_dep->is_bool)
+		if (temp_dep->type != type)
 			goto __skip;
 		card_name=get_card_name(temp_dep->name);
 		if (card_name) {
@@ -1060,7 +1086,7 @@ static void sel_print_acinclude(struct sel *sel)
 				printf(" && ");
 			}
 			printf("CONFIG_%s=\"%c\"\n", nsel->name,
-				(nsel->dep && nsel->dep->is_bool) ? 'y' : 'm');
+				(nsel->dep && nsel->dep->type == TYPE_BOOL) ? 'y' : 'm');
 		}
 	}
 }
@@ -1077,7 +1103,7 @@ static void output_acinclude(void)
 	
 	printf("dnl ALSA soundcard configuration\n");
 	printf("dnl Find out which cards to compile driver for\n");
-	printf("dnl Copyright (c) by Jaroslav Kysela <perex@suse.cz>,\n");
+	printf("dnl Copyright (c) by Jaroslav Kysela <perex@perex.cz>,\n");
 	printf("dnl                  Anders Semb Hermansen <ahermans@vf.telia.no>\n\n");
 
 	printf("AC_DEFUN([ALSA_TOPLEVEL_INIT], [\n");
@@ -1096,7 +1122,7 @@ static void output_acinclude(void)
 	       "  [                        cards may be separated with commas; ]\n"
 	       "  [                        'all' compiles all drivers; ]\n"
 	       "  [                        Possible cards are: ]\n");
-	output_card_list(all_deps, 26, 50, 1);
+	output_card_list(all_deps, 26, 50, TYPE_TRISTATE);
 	printf(" ],\n");
 	printf("  cards=\"$withval\", cards=\"all\")\n");
 	printf("SELECTED_CARDS=`echo $cards | sed 's/,/ /g'`\n");
@@ -1105,7 +1131,7 @@ static void output_acinclude(void)
 	       "  probed=\n");
 	for (tempdep = all_deps; tempdep; tempdep = tempdep->next) {
 		int put_if;
-		if (!is_toplevel(tempdep) || tempdep->is_bool)
+		if (!is_toplevel(tempdep) || tempdep->type != TYPE_TRISTATE)
 			continue;
 		text = get_card_name(tempdep->name);
 		if (! text)
@@ -1161,10 +1187,13 @@ static void output_acinclude(void)
 				printf(" && ");
 			}
 			printf("CONFIG_%s=\"%c\"\n", sel->name,
-			       (sel->dep && sel->dep->is_bool) ? 'y' : 'm');
+			       (sel->dep && sel->dep->type == TYPE_BOOL) ? 'y' : 'm');
 		}
 		text = convert_to_config_uppercase("CONFIG_", tempdep->name);
-		printf("      %s=\"%c\"\n", text, tempdep->is_bool ? 'y' : 'm');
+		if (tempdep->type == TYPE_INT)
+			printf("      %s=\"%d\"\n", text, tempdep->int_val);
+		else
+			printf("      %s=\"%c\"\n", text, tempdep->type == TYPE_BOOL ? 'y' : 'm');
 		free(text);
 		printf("      probed=1\n");
 		if (put_if)
@@ -1185,7 +1214,7 @@ static void output_acinclude(void)
 	       "  [                        options may be separated with commas; ]\n"
 	       "  [                        'all' enables all options; ]\n"
 	       "  [                        Possible options are: ]\n");
-	output_card_list(all_deps, 26, 50, 0);
+	output_card_list(all_deps, 26, 50, TYPE_BOOL);
 	printf(" ],\n");
 	printf("  cards=\"$withval\", cards=\"all\")\n");
 	printf("SELECTED_OPTIONS=`echo $cards | sed 's/,/ /g'`\n");
@@ -1194,7 +1223,7 @@ static void output_acinclude(void)
 	       "  probed=\n");
 	for (tempdep = all_deps; tempdep; tempdep = tempdep->next) {
 		int put_if;
-		if (!is_toplevel(tempdep) || !tempdep->is_bool)
+		if (!is_toplevel(tempdep) || tempdep->type == TYPE_TRISTATE)
 			continue;
 		text = get_card_name(tempdep->name);
 		if (! text)
@@ -1226,6 +1255,8 @@ static void output_acinclude(void)
 		for (sel = tempdep->sel; sel; sel = sel->next)
 			sel_print_acinclude(sel);			
 		for (sel = tempdep->sel; sel; sel = sel->next) {
+			if (sel->dep->type == TYPE_INT)
+				continue;
 			if (is_always_true(sel->dep))
 				continue;
 			printf("      ");
@@ -1235,10 +1266,13 @@ static void output_acinclude(void)
 				printf(" && ");
 			}
 			printf("CONFIG_%s=\"%c\"\n", sel->name,
-			       (sel->dep && sel->dep->is_bool) ? 'y' : 'm');
+			       (sel->dep && sel->dep->type == TYPE_BOOL) ? 'y' : 'm');
 		}
 		text = convert_to_config_uppercase("CONFIG_", tempdep->name);
-		printf("      %s=\"y\"\n", text);
+		if (tempdep->type == TYPE_INT)
+			printf("      %s=\"%d\"\n", text, tempdep->int_val);
+		else
+			printf("      %s=\"y\"\n", text);
 		free(text);
 		printf("      probed=1\n");
 		if (put_if)
@@ -1264,7 +1298,10 @@ static void output_acinclude(void)
 			continue;
 		text = convert_to_config_uppercase("CONFIG_", tempdep->name);
 		printf("if test -n \"$%s\"; then\n", text);
-		if (tempdep->is_bool)
+		if (tempdep->type == TYPE_INT) {
+			printf("  AC_DEFINE_UNQUOTED([%s], [%d])\n",
+			       text, tempdep->int_val);
+		} else if (tempdep->type == TYPE_BOOL)
 			printf("  AC_DEFINE(%s)\n", text);
 		else
 			printf("  AC_DEFINE(%s_MODULE)\n", text);
@@ -1290,7 +1327,7 @@ static void output_makeconf(void)
 	char *text;
 	
 	printf("# Soundcard configuration for ALSA driver\n");
-	printf("# Copyright (c) by Jaroslav Kysela <perex@suse.cz>,\n");
+	printf("# Copyright (c) by Jaroslav Kysela <perex@perex.cz>,\n");
 	printf("#                  Anders Semb Hermansen <ahermans@vf.telia.no>\n\n");
 	for (tempdep = all_deps; tempdep; tempdep = tempdep->next) {
 		text = convert_to_config_uppercase("CONFIG_", tempdep->name);
@@ -1306,12 +1343,12 @@ static void output_include(void)
 	char *text;
 	
 	printf("/* Soundcard configuration for ALSA driver */\n");
-	printf("/* Copyright (c) by Jaroslav Kysela <perex@suse.cz>, */\n");
+	printf("/* Copyright (c) by Jaroslav Kysela <perex@perex.cz>, */\n");
 	printf("/*                  Anders Semb Hermansen <ahermans@vf.telia.no> */\n\n");
 	for (tempdep = all_deps; tempdep; tempdep = tempdep->next) {
 		text = convert_to_config_uppercase("CONFIG_", tempdep->name);
 		printf("#undef %s%s\n", text,
-		       tempdep->is_bool ? "" : "_MODULE");
+		       tempdep->type == TYPE_TRISTATE ? "_MODULE" : "");
 		free(text);
 	}
 }
